/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton.model;

import org.xmloperator.java.skeleton.model.ClassAndInterfaceList;
import org.xmloperator.java.skeleton.model.Package;

/**
 * A ClassOrInterface object represents a Java class or interface.
 * Its declaration may or may not be encountered.
 */
public interface ClassOrInterface {

  /**
   * Return the parent package. Not null.
   */
  public Package getPackage();

  /**
   * Return the class or interface identifier. Not null neither empty.
   * The identifier is unique among the classes of the package.
   */
  public String getIdentifier();

  /**
   * Return the class or interface qualified identifier. Not null neither empty.
   * If the parent package is identified then the qualified identifier is :
   *   <parent package qualified identifier> . <identifier>
   * If the parent package is a default package then the qualified identifier
   *   is the identifier.
   */
  public String getQualifiedIdentifier();

  /**
   * Return true if the class or interface declaration has been met.
   * Return false if the class or interface is only identified by a using
   *   class or interface.
   */
  public boolean isDefined();

  /**
   * Makes the class defined. The class declaration has been met.
   */
  public void makeDefined();

  /**
   * Return the list of used classes and interfaces.
   */
  public ClassAndInterfaceList getUsedClassAndInterfaceList();

  /**
   * Return the list of classes and interfaces that use this one.
   */
  public ClassAndInterfaceList getUserClassAndInterfaceList();
}
