/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton.util;

import java.io.PrintWriter;
import java.util.Collections;
import java.util.Stack;
import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.ArrayList;

import org.xmloperator.java.skeleton.model.ClassOrInterface;
import org.xmloperator.java.skeleton.model.ClassAndInterfaceList;
import org.xmloperator.java.skeleton.model.Package;
import org.xmloperator.java.skeleton.model.IdentifiedPackage;
import org.xmloperator.java.skeleton.model.Skeleton;

/**
 * Write a Skeleton object in XML form.
 */
public final class SkeletonWriter {
  private static final String TAG_USES = "uses";
  private static final String TAG_USEDBY = "usedBy";
  private PrintWriter out;
  private boolean withReverseUse;
  private boolean withPackageUse;
  private final String tab = "  ";
  private final Stack nameStack = new Stack();

  public static final void printSkeleton(
      PrintWriter out, boolean withReverseUse, boolean withPackageUse,
      Skeleton skeleton) {
    new SkeletonWriter(out, withReverseUse, withPackageUse).
        printSkeleton(skeleton);
  }

  private SkeletonWriter(PrintWriter out,
                         boolean withReverseUse, boolean withPackageUse) {
    this.out = out;
    this.withReverseUse = withReverseUse;
    this.withPackageUse = withPackageUse;
  }

  //////////////////////////
  // Skeleton main structure
  //////////////////////////

  private void printSkeleton(Skeleton skeleton) {
    this.startTagStart("javaSkeleton");
    this.attribute("xmlns", "http://www.xmloperator.net/namespace/java");
    this.startTagEnd();
    this.startTag("import");
    this.printSkeleton(skeleton, true);
    this.endTag();
    this.printSkeleton(skeleton, false);
    this.endTag();
  }

  private void printSkeleton(Skeleton skeleton, boolean isImport) {
    this.printDefaultPackage(skeleton.getDefaultPackage(), isImport);
    for (int k = 0; k < skeleton.getPrimaryPackageCount(); k++)
      this.printIdentifiedPackage(skeleton.getPrimaryPackage(k), isImport);
  }

  private void printDefaultPackage(Package defaultPackage, boolean isImport) {
    if (!defaultPackage.hasClassOrInterface(!isImport))
      return;
    this.startTag("defaultPackage");
    this.printPackage(defaultPackage, isImport);
    if (this.withPackageUse && !isImport) {
      Set usedTypes = new HashSet();
      Set userTypes = this.withReverseUse ? new HashSet() : null;
      addPackageUse(usedTypes, userTypes, defaultPackage);
      this.printPackageUse(TAG_USES, usedTypes);
      if (this.withReverseUse)
        this.printPackageUse(TAG_USEDBY, userTypes);
    }
    this.endTag();
  }

  private void printIdentifiedPackage(IdentifiedPackage identifiedPackage,
                                      boolean isImport) {
    if (!identifiedPackage.hasClassOrInterface(!isImport))
      return;
    this.startTagStart("package");
    this.attribute("identifier", identifiedPackage.getIdentifier());
    this.attribute("qualifiedIdentifier",
                   identifiedPackage.getQualifiedIdentifier());
    this.startTagEnd();
    for (int k = 0; k < identifiedPackage.getChildPackageCount(); k++)
      this.printIdentifiedPackage(identifiedPackage.getChildPackage(k),
                                  isImport);
    this.printPackage(identifiedPackage, isImport);
    if (this.withPackageUse && !isImport) {
      Set usedTypes = new HashSet();
      Set userTypes = this.withReverseUse ? new HashSet() : null;
      addIdentifiedPackageUse(usedTypes, userTypes, identifiedPackage);
      this.printPackageUse(TAG_USES, usedTypes);
      if (this.withReverseUse)
        this.printPackageUse(TAG_USEDBY, userTypes);
    }
    this.endTag();
  }

  private void printPackage(Package aPackage, boolean isImport) {
    for (int k = 0; k < aPackage.getClassAndInterfaceCount(); k++) {
      ClassOrInterface classOrInterface = aPackage.getClassOrInterface(k);
      if (isImport ^ classOrInterface.isDefined())
        this.printClassOrInterface(classOrInterface);
    }
  }

  private void printClassOrInterface(ClassOrInterface classOrInterface) {
    this.startTagStart("classOrInterface");
    this.attribute("identifier", classOrInterface.getIdentifier());
    this.attribute("qualifiedIdentifier",
                   classOrInterface.getQualifiedIdentifier());
    this.startTagEnd();
    this.printClassAndInterfaceList(
        TAG_USES, classOrInterface.getUsedClassAndInterfaceList());
    if (this.withReverseUse)
      this.printClassAndInterfaceList(
          TAG_USEDBY, classOrInterface.getUserClassAndInterfaceList());
    this.endTag();
  }

  private void printClassAndInterfaceList(String tag,
      ClassAndInterfaceList classAndInterfaceList) {
    if (classAndInterfaceList.getClassAndInterfaceCount() == 0)
      return;
    this.startTag(tag);
    for (int k = 0;
         k < classAndInterfaceList.getClassAndInterfaceCount(); k++) {
      this.startTagStart("type");
      this.attribute(
          "qualifiedIdentifier",
          classAndInterfaceList.getClassOrInterface(k).getQualifiedIdentifier());
      this.startTagEnd();
      this.endTag();
    }
    this.endTag();
  }

  //////////////
  // Package use
  //////////////

  private static void addIdentifiedPackageUse(
      Set usedTypes, Set userTypes, IdentifiedPackage identifiedPackage) {
    addPackageUse(usedTypes, userTypes, identifiedPackage);
    for (int k = 0; k < identifiedPackage.getChildPackageCount(); k++)
      addIdentifiedPackageUse(
          usedTypes, userTypes, identifiedPackage.getChildPackage(k));
  }

  private static void addPackageUse(Set usedTypes, Set userTypes,
                                    Package packag) {
    for (int k = 0; k < packag.getClassAndInterfaceCount(); k++) {
      ClassOrInterface classOrInterface = packag.getClassOrInterface(k);
      addClassOrInterfaceList(
          usedTypes, classOrInterface.getUsedClassAndInterfaceList());
      if (userTypes != null)
        addClassOrInterfaceList(
            userTypes, classOrInterface.getUserClassAndInterfaceList());
    }
  }

  private static void addClassOrInterfaceList(
      Set types, ClassAndInterfaceList classOrInterfaceList) {
    for (int k = 0; k < classOrInterfaceList.getClassAndInterfaceCount(); k++)
      types.add(classOrInterfaceList.getClassOrInterface(k));
  }

  private void printPackageUse(String tag, Set types) {
    if (types.isEmpty())
      return;
    this.startTag(tag);
    List typeQualifiedIdentifierList = new ArrayList();
    Iterator iterator = types.iterator();
    while (iterator.hasNext()) {
      ClassOrInterface type = (ClassOrInterface)iterator.next();
      typeQualifiedIdentifierList.add(type.getQualifiedIdentifier());
    }
    Collections.sort(typeQualifiedIdentifierList);
    for (int k = 0; k < typeQualifiedIdentifierList.size(); k++) {
      this.startTagStart("type");
      this.attribute("qualifiedIdentifier",
                     (String)typeQualifiedIdentifierList.get(k));
      this.startTagEnd();
      this.endTag();
    }
    this.endTag();
  }

  ////////////////////
  // Low level methods
  ////////////////////

  private void startTagStart(String name) {
    this.tab();
    out.print('<');
    out.print(name);
    this.nameStack.push(name);
  }

  private void attribute(String name, String value) {
    out.print(' ');
    out.print(name);
    out.print("=\"");
    out.print(value);
    out.print('"');
  }

  private void startTagEnd() {
    out.println('>');
  }

  private void startTag(String name) {
    this.startTagStart(name);
    this.startTagEnd();
  }

  private void emptyTag(String name) {
    this.tab();
    out.print('<');
    out.print(name);
    out.println("/>");
  }

  private void endTag() {
    if (this.nameStack.isEmpty())
      throw new IllegalStateException();
    String name = (String)nameStack.pop();
    this.tab();
    out.print("</");
    out.print(name);
    out.println('>');
  }

  private void emptyTagEnd() {
    if (this.nameStack.isEmpty())
      throw new IllegalStateException();
    this.nameStack.pop();
    out.println("/>");
  }

  private void tab() {
    for (int k = 0; k < this.nameStack.size(); k++)
      out.print(this.tab);
  }
}
