/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.syntax;

import java.util.List;

/**
 * The purpose of this interface is to notify class and interface declaration
 *   and referencing within java source compilation units.
 * The notifier is usually a Java compilation unit parser.
 * The consumer (which implements the interface) is usually concerned by
 *   class and interface dependencies.
 */
public interface CompilationUnitHandler {

  /**
   * Notify the beginning of a compilation unit,
   *   which will be called the current compilation unit.
   * Can occurs only if there is no current compilation unit.
   * Provide the package qualified identifier.
   *   It is null for the default package.
   */
  public void startCompilationUnit(String packageQualifiedIdentifier);

  /**
   * Notify the end of the current compilation unit.
   * Can occur only if there is a current compilation unit.
   */
  public void endCompilationUnit();

  /**
   * Notify the beginning of a outer class,
   *   which will be called the current class.
   * Can occur only if there is a current compilation unit and there is no
   *   current class or interface.
   * Provide the class identifier (not null neither empty), the qualified
   *   identifier of the extended class if any (null elsewhere) and the list
   *   of the qualified identifiers of implemented interfaces.
   */
  public void startClass(String classIdentifier,
                         String extendedClassQualifiedIdentifier,
                         List implementedInterfaceQualifiedIdentifierList);

 /**
  * Notify the beginning of a outer interface,
  *   which will be called the current interface.
  * Can occur only if there is a current compilation unit and there is no
  *   current class or interface.
  * Provide the interface identifier (not null neither empty) and the list
  *   of the qualified identifiers of extended interfaces.
  */
  public void startInterface(String interfaceIdentifier,
                             List extendedInterfaceQualifiedIdentifierList);

 /**
  * Notify the end of the current class or interface.
  * Can occur only if there is a current class or interface.
  */
  public void endClassOrInterface();

  /**
   * Notify a class or interface referencing.
   * Include the declaration of field or method, the using of a class
   *   or an interface reference within an expression such as instanceof
   *   or a casted expression.
   * Exclude inner class or interface referencing and instanciation.
   * Can occur only if there is a current class or interface.
   * Provide the qualified identifier of the referenced class or interface.
   */
  public void classOrInterfaceReference(
      String classOrInterfaceQualifiedIdentifier);

  /**
   * Notify the instanciation of a outer class using a *new* statement.
   * Can occur only if there is a current class.
   * Provide the qualified identifier of the instanciated class.
   */
  public void instanciation(String classQualifiedIdentifier);
}
